package javapas;

import java.util.*;

/**
 * <p>Title: Javapas</p>
 * <p>Description: Pascal Interpreter</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: AP</p>
 *
 * Klasa odpowiedzialna jest za stworzenie pseudokodu wykonywalnego
 * przez maszyn wirtualn VM z kodu rdowego w jzyku Pascal.
 *
 * @author Wojciech Nabiaek
 * @version 1.0
 */

public class Prekompilator {
  /** Wektor zawierajcy nazwy zmiennych */
  private Vector zmienne = new Vector();
  /** Wektor zawierajcy list operacji (pseudokod) */
  private Vector operacje = new Vector();
  /** Kod rdowy programu w jzyku Pascal */
  private String zrodlo = "";

  /**
   * Konstrukor klasy wymaga podania w parametrze
   * acucha znakw zawierajcego peen kod rdowy programu.
   *
   * @param Zrodlo kod rdowy programu.
   */
  public Prekompilator(String Zrodlo) {
    zrodlo=Zrodlo;
  }

  /**
   * Funkcja zwracajca poprawnie sformatowany typ zmiennej.
   *
   * @param typ nazwa typu
   * @return poprawnie sformatowan nazw typu, jeli paramert typ nie jest typem pascala funkcja zwraca warto "".
   */
  private String normalizujTyp(String typ){
    String wy = "";
    if (typ.equalsIgnoreCase("boolean")) wy="Boolean";
    if (typ.equalsIgnoreCase("integer")) wy="Integer";
    if (typ.equalsIgnoreCase("string")) wy="String";
    if (typ.equalsIgnoreCase("real")) wy="Real";
    return wy;
  }

  /**
   * Metoda usuwa z kodu rdowego wszystkie komentarze tj., znaki zamieszczone wewntrz klamer {}.
   */
  private void usunKomentarze(){
    for( ; zrodlo.indexOf("{")>=0; ){
      int otw=zrodlo.indexOf("{");
      int zmk=zrodlo.indexOf("}");
      String przed = zrodlo.substring(0, otw);
      String za    = zrodlo.substring(zmk+1);
      zrodlo       = przed+za;
    };
  }

  /**
   * Metoda usuwa z kodu rdowego wszystkie znaki ^, bdce z punktu widzenia tego moduu interpretera cakowicie zbyteczne.
   */
  private void usunWskazniki(){
    String wy = "";
    for(int i=0; i<zrodlo.length(); i++){
      if(!(zrodlo.charAt(i)=='^' && !czyWStringu(zrodlo,i))) wy+=zrodlo.charAt(i);
    };
    zrodlo=wy;
  }

  /**
   * Do poprawnego dziaania moduu wymagane jest aby przed sowem kluczowym <code>else</code> wystpowa rednik, co jest niezgodne ze skadni jzyka Pascal.<br>
   * Funkcja ta dodaje na potrzeby klasy redniki przed kadym wystpieniem sowa <code>else</code>.
   */
  private void normalizujElse(){
    String pom = new String(zrodlo);
    for(int i=0; ;){
      int poz = pom.toLowerCase().indexOf("else");
      if(poz==-1) break;
      i+=poz;
      if(!czyWStringu(pom, poz)){
        String lewa = zrodlo.substring(0, i).trim();
        if(lewa.charAt(lewa.length()-1)!=';') lewa+="; ";
        zrodlo = lewa + zrodlo.substring(i).trim();
        i+=2;
        pom = pom.substring(poz+4);
      } else
        pom = ("'"+pom.substring(poz+3));
//      i+=4;
    }
  }

  /**
   * Metoda wczytuje blok <code>var</code> oraz nastpnie dodaje je do wektora zmiennych.
   */
  private void dajZmienne(){
    String zr = zrodlo.toLowerCase();
    int varPoz=zr.indexOf("var");
    if(varPoz>=0){
      int nextPoz = min(zr.indexOf("const"), zr.indexOf("begin"), varPoz);
      String zm = zr.substring(varPoz+3, nextPoz).trim();
      for (; zm.trim().length()>0; ){
        String linia = zm.substring(0, zm.indexOf(";")).trim();
        String typ = normalizujTyp(linia.substring(linia.indexOf(":")+1).trim());
        linia = linia.substring(0, linia.indexOf(":")).trim();
        for (; linia.trim().length()>0; ){
          int pozPrzec = linia.indexOf(",");
          if(pozPrzec<0){
            zmienne.add(new Zmienna(linia, typ));
            linia="";
          } else {
            String nazwa = linia.substring(0, pozPrzec).trim();
            linia = linia.substring(pozPrzec+1).trim();
            zmienne.add(new Zmienna(nazwa, typ));
          }
        }
        zm=zm.substring(zm.indexOf(";")+1).trim();
      }
      zm="";
    }
    zr="";
  }

  /**
   * Metoda wczytuje blok <code>const</code> oraz nastpnie dodaje je do wektora zmiennych z zaznaczeniem bitu staoci.
   */
  private void dajStale(){
    String zr = zrodlo.toLowerCase();
    int varPoz=zr.indexOf("const");
    if(varPoz>=0){
      int nextPoz = min(zr.indexOf("var"), zr.indexOf("begin"), varPoz);
      String zm = zr.substring(varPoz+5, nextPoz);
      for (; zm.trim().length()>0; ){
        String linia = zm.substring(0, zm.indexOf(";")).trim();
        String wartosc = linia.substring(linia.indexOf("=")+1).trim();
        linia = linia.substring(0, linia.indexOf("=")).trim();
        zmienne.add(new Zmienna(linia, wartosc, true));
        zm=zm.substring(zm.indexOf(";")+1).trim();
      }
      zm="";
    }
    zr="";
  }

  /**
   * Metoda wczytuje z kodu rdowego programu blok gwny <code>Begin ... End.</code> a nastpnie wycina z tego bloku
   * wiodce <code>Begin</code> i koczce <code>End.</code>
   *
   * @return wntrze bolku gwnego programu.
   */
  private String dajBlokGlowny(){
    String zr = zrodlo.toLowerCase();
    int begin = zr.indexOf("begin")+5;
    int end = zr.indexOf("end.");
    return zrodlo.substring(begin, end).trim();
  }

  /**
   * Funkcja zwraca wntrze bloku <code>Begin</code> ... <code>End;</code> dowolnej operacji czy instrukcji.
   * @param kod kod rdowy programu.
   * @return wntrze bloku <code>Begin</code> ... <code>End;</code>
   */
  private Blok dajBlokOperacji(String kod){
    Blok wy = new Blok();
    wy.kod=kod;
    kod = kod.replaceAll("begin", "{");
    kod = kod.replaceAll("end", "} ");
    int pierw = kod.indexOf("{");
    int ost = kod.length();
    int liczba = 1;
    for(int i=pierw+1; liczba>0 && i<kod.length(); i++){
      if(kod.charAt(i)=='{') liczba++;
      if(kod.charAt(i)=='}') liczba--;
      if(liczba==0) {
        ost=i;
        break;
      };
    }
    kod = kod.substring(pierw+1, ost).trim();
    wy.koniec=ost+((liczbaWystapien(kod, "{")+1)*4)+((liczbaWystapien(kod, "}")+1)*2);
    if(kod.indexOf("{")>-1) kod = replaceAll('{', kod, "begin");
    if(kod.indexOf("}")>-1) kod = replaceAll('}', kod, "end");
    wy.blok=kod;
    return wy;
  }

  /**
   * Metoda zamienia w acuchu <code>gdzie</code> wszystkie wystpienia znaku <code>co</code>
   * na acuch znakw <code>naCo</code>
   *
   * @param co znak jaki chcemy zamieni
   * @param gdzie acuch znakw, w ktrym dokonujemy zmian
   * @param naCo acuch znakw na jaki zamieniamy znak <code>co</code>
   * @return acuch znakw po zamianie
   */
  private String replaceAll(char co, String gdzie, String naCo){
    String wy = "";
    int lp=0;
    for (int i=0; i<gdzie.length(); i++){
      if (gdzie.charAt(i)==co){
        wy+=gdzie.substring(0, i);
        wy+=naCo;
        i++;
        lp=i;
      }
    }
    wy+=gdzie.substring(lp);
    return wy;
  }

  /**
   * Metoda odpowiedzialna za kompilacje kodu rdowego znajdujcego si wewntrz bloku instrukcji.
   *
   * @param kod kod rdowy bloku programu.
   */
  private void wykonajBlok(String kod){
    for(; kod.trim().length()>0; )
      kod = wiersz(kod);
  }

  /**
   * Metoda wykonuje interpretacj pierwszego wiersza z kodu rdowego a
   * nastpnie odcina wykonan cz z kodu rdowego.
   *
   * @param kod kod rdowy programu.
   * @return kod rdowy programu pozbawiony zinterpretowanej czci.
   */
  private String wiersz(String kod){
    String km = kod.toLowerCase();
    int koniec = min(min(pierwszyRaz(kod, ";"), pierwszyRaz(kod, "then"), -1), min(pierwszyRaz(kod, "do"), pierwszyRaz(kod, "begin"), -1), -1);
    if(kod.charAt(koniec)==';') {
      instrukcja(kod.substring(0, koniec).trim());
      kod=kod.substring(koniec+1).trim();
    } else if(kod.charAt(koniec)=='t') {
      kod=IF(kod.substring(2, koniec).trim(), kod.substring(koniec+4).trim()).trim();
//      kod=kod.substring(koniec+4).trim();
    } else if(kod.charAt(koniec)=='d') {
      kod=FOR(kod.substring(3, koniec).trim(), kod.substring(koniec+2).trim()).trim();
    } else if(kod.charAt(koniec)=='b') {
      Blok b = dajBlokOperacji(kod.trim());
      String wnetrze = b.blok;
      wykonajBlok(wnetrze);
      kod = kod.substring(b.koniec+2);
    }
    km="";
    return kod;
  }

  /**
   * Metoda rozpoznaje i zamienia na pseudokod instrukcj podstawienia bd znane funkcje.
   *
   * @param kod rdowy programu.
   */
  private void instrukcja(String kod){
    int podst = pierwszyRaz(kod, ":=");
    if(podst>0) { // Instrukcja podstawienia
      String lewa = kod.substring(0, podst);
      String prawa = kod.substring(podst+2);
      operacje.addElement(":=");
      operacje.addElement("$"+indexZmiennej(lewa));
      wyrazenie(prawa);
    } else { //wywolanie procedury
      int nawias = pierwszyRaz(kod, "(");
      if(nawias<0){ //procedura bez parametrow
        if(kod.equalsIgnoreCase("writeln")) operacje.addElement(kod);
      } else { //procedura z parametrami
        String proc = kod.substring(0, nawias).trim();
        String parametry = dajNawiasy(kod);
        if(proc.equalsIgnoreCase("write")) write(parametry);
        if(proc.equalsIgnoreCase("writeln")) writeln(parametry);
        if(proc.equalsIgnoreCase("read")) read(parametry);
        if(proc.equalsIgnoreCase("readln")) readln(parametry);
        // Tu mozna dodawac kolejne obslugiwane procedury //
      }
    }
  }

  /**
   * Metoda zamienia na pseudokod wyraenie logiczne.
   *
   * @param kod rdowy programu.
   */
  private void wyrazenie(String kod){
    int or  = pierwszyRaz(kod, "or");
    int and = pierwszyRaz(kod, "and");
    if( or>-1 && or>and ) { //Mamy binarne OR na koncu wyrazenia
      operacje.addElement("or");
      wyrazenie(kod.substring(0, or).trim());
      wyrazenie(kod.substring(or+2).trim());
    } else if(and>-1 && and>or) { //Mamy binarne OR na koncu wyrazenia
      operacje.addElement("and");
      wyrazenie(kod.substring(0, and).trim());
      wyrazenie(kod.substring(and+3).trim());
    } else { //Nie ma operatorw binarnych wiec lecimy dalej
      int mnr=pierwszyRaz(kod, "<=");
      int wr =pierwszyRaz(kod, ">=");
      int mw =pierwszyRaz(kod, "<>");
      int r  =pierwszyRaz(kod, "=");
      int m  =pierwszyRaz(kod, "<");
      int w  =pierwszyRaz(kod, ">");
      if(mnr>-1) { //Mamy operator <=
        operacje.addElement("<=");
        wyrazenie(kod.substring(0, mnr).trim());
        wyrazenie(kod.substring(mnr+2).trim());
      } else if(wr>-1) { //Mamy operator >=
        operacje.addElement(">=");
        wyrazenie(kod.substring(0, wr).trim());
        wyrazenie(kod.substring(wr+2).trim());
      } else if(mw>-1) { //Mamy operator <>
        operacje.addElement("<>");
        wyrazenie(kod.substring(0, mw).trim());
        wyrazenie(kod.substring(mw+2).trim());
      } else if(r>-1) { //Mamy operator =
        operacje.addElement("=");
        wyrazenie(kod.substring(0, r).trim());
        wyrazenie(kod.substring(r+1).trim());
      } else if(m>-1) { //Mamy operator <
        operacje.addElement("<");
        wyrazenie(kod.substring(0, m).trim());
        wyrazenie(kod.substring(m+1).trim());
      } else if(w>-1) { //Mamy operator >
        operacje.addElement(">");
        wyrazenie(kod.substring(0, w).trim());
        wyrazenie(kod.substring(w+1).trim());
      } else { //Nie ma juz operatorow porownania
        int plus  = pierwszyRaz(kod, "+");
        int minus = pierwszyRaz(kod, "-");
        int razy  = pierwszyRaz(kod, "*");
        int podz  = pierwszyRaz(kod, "/");
        if(razy>-1) { //Mamy mnozenie
          operacje.addElement("*");
          wyrazenie(kod.substring(0, razy).trim());
          wyrazenie(kod.substring(razy+1).trim());
        } else if(podz>-1){ //Mamy dzielenie
          operacje.addElement("/");
          wyrazenie(kod.substring(0, podz).trim());
          wyrazenie(kod.substring(podz+1).trim());
        } else if(plus>-1){ //Mamy dodawanie
          operacje.addElement("+");
          wyrazenie(kod.substring(0, plus).trim());
          wyrazenie(kod.substring(plus+1).trim());
        } else if(minus>-1){ //Mamy odejmowanie
          operacje.addElement("-");
          wyrazenie(kod.substring(0, minus).trim());
          wyrazenie(kod.substring(minus+1).trim());
        } else { //Nie ma wiecej operacji arytmerycznych
          int zm = indexZmiennej(kod);
          if(zm>-1) { //Mamy zmienn
            operacje.addElement("$"+zm);
          } else { //Mamy jak warto
            operacje.addElement(kod);
          }
        }
      }
    }
  }

  /**
   * Metoda odpowiedzialna za pobranie wntrza nawiasw ()
   *
   * @param kod rdowy programu
   * @return wntrze nawiasw
   */
  private String dajNawiasy(String kod){
    int pierw = pierwszyRaz(kod, "(");
    int ost   = pierwszyRaz(kod, ")");
    int liczba = 1;
    for(int i=pierw+1; liczba>0 && i<kod.length(); i++){
      if(kod.charAt(i)=='(') liczba++;
      if(kod.charAt(i)==')') liczba--;
      if(liczba==0) {
        ost=i;
        break;
      };
    }
    return kod.substring(pierw+1, ost).trim();
  }

  /**
   * Metoda odpowiedzialna za pobranie wntrza apostrofw ''
   *
   * @param kod rdowy programu
   * @return wntrze apostrofw ''
   */
  private String dajString(String kod){
    int pierw = kod.indexOf("'");
    String pom = kod.substring(pierw+1);
    int ost   = pom.indexOf("'");
    return pom.substring(0, ost);
  }

  /**
   * Metoda sprawdza czy znak o indeksie poz znajduje si wewntrz apostrofw ''.
   *
   * @param kod rdowy programu.
   * @param poz numer indeksu znaku.
   * @return <code>true</code> jeli indeks znajduje si wewntrz apostrofw, <code>false</code> w przeciwnym przypadku
   */
  private boolean czyWStringu(String kod, int poz){
/*    int pierw = kod.indexOf("'");
    String pom = kod.substring(pierw+1);
    int ost   = pom.indexOf("'");
    if(poz>pierw && poz<ost) return true;
    else return false;
*/
    boolean pocz=false;
    char [] pom = kod.toCharArray();
    for(int i=0; i<kod.length(); i++){
      if (pom[i]=='\'') pocz=!pocz;
      if (pocz) pom[i]=0;
    }
    if(poz>-1 && poz<kod.length() && pom[poz]==0) return true;
    else return false;
  }

  /**
   * Metoda wyszukuje pierwszego wystpienia acucha znakw <code>co</code> w acuchu <code>kod</code>
   * pod warunkiem, e nie wystpuje ono w tekcie ograniczonym apostrofami.
   *
   * @param kod kod rdowy w Pascalu
   * @param co szukany tekst
   * @return indeks pocztku szukanego tekstu w kodzie rdowym
   */
  private int pierwszyRaz(String kod, String co){
    int wy=-1;
    for(; kod.trim().length()>0;){
      int i = kod.toLowerCase().indexOf(co.toLowerCase());
      if(czyWStringu(kod, i)) kod = kod.substring(i+co.length());
      else return i;
    }
    return wy;
  }

  /**
   * Metoda zlicza licz symboli <code>+</code> w acuchu znakw.
   * @param gdzie acuch znakw do przeszukania.
   * @return liczba symboli <code>+</code>
   */
  private int liczbaPlusow(String gdzie){
    int wy=0;
    for(int i=0; i<gdzie.length(); i++)
      if(gdzie.charAt(i)=='+')
        wy++;
    return wy;
  }

  /**
   * Metoda zlicza liczb wystpie acucha znakw <code>co</code> w acuchu <code>gdzie</code>.
   *
   * @param gdzie acuch znakw do przeszukania
   * @param co zliczany acuch znakw
   * @return liczba wystpie
   */
  private int liczbaWystapien(String gdzie, String co){
    int wy=0;
    for(; gdzie.length()>0; )
      if(gdzie.indexOf(co)>-1){
        wy++;
        gdzie = gdzie.substring(gdzie.indexOf(co)+co.length());
      } else gdzie="";
    return wy;
  }

  /**
   * Metoda tworzy pseudokod instrukcji warunkowej <code>IF ... THEN ... ELSE</code>
   *
   * @param warunek warunek wejcia do instrukcji <code>IF</code>
   * @param kod kod rdowy programu poczwszy od koca instrukcji <code>THEN</code>
   * @return kod rdowy pozbawiony zinterpretowanej czci.
   */
  private String IF(String warunek, String kod){
    operacje.addElement("if");
    int koniec = operacje.size();
    operacje.addElement(" - tu bedzie adres konca ifa (ew. poczatek else) - ");
    wyrazenie(warunek);
    if(kod.length()>5 && kod.trim().substring(0,5).equalsIgnoreCase("begin")){ // Mamy blok operacji
      Blok b = new Blok();
      b = dajBlokOperacji(kod.trim());
      String wnetrze = b.blok;
      wykonajBlok(wnetrze);
      if(kod.length()>b.koniec+2) kod = kod.substring(b.koniec+2).trim();
      else  kod = kod.substring(b.koniec).trim();
    } else { // Mamy pojedycz operacj
      kod = wiersz(kod).trim();
    }
    operacje.addElement("jmp");
    int kc = operacje.size();
    operacje.addElement(" - tu bedzie koniec claego bloku if ... else - ");
    operacje.setElementAt(String.valueOf(operacje.size()), koniec); // zaznaczenie konca if i poczatku ew. else
    // Musimy znalezc (lub nie) else
    if(kod.length()>4 && kod.trim().substring(0,4).equalsIgnoreCase("else")){ // Mamy else
      kod = kod.trim().substring(4).trim();
      if(kod.length()>5 && kod.trim().substring(0,5).equalsIgnoreCase("begin")){ // Mamy blok operacji
        Blok b = new Blok();
        b = dajBlokOperacji(kod.trim());
        String wnetrze = b.blok;
        wykonajBlok(wnetrze);
        if(kod.length()>b.koniec+2) kod = kod.substring(b.koniec+2).trim();
        else  kod = kod.substring(b.koniec).trim();
      } else { // Mamy pojedycz operacj
        kod = wiersz(kod);
      }
    }
    // Ostatnia operacja tej funkcji...
    operacje.setElementAt(String.valueOf(operacje.size()), kc);
    return kod;
  }

  /**
   * Metoda tworzy pseudokod ptli <code>FOR ... TO ... DO</code>
   *
   * @param warunek warunek pocztku ptli oraz jej koczenia <code>... TO ...</code> (bez sw <code>FOR</code> i <code>DO</code>
   * @param kod kod rdowy programu poczwszy od koca instrukcji <code>DO</code>
   * @return kod rdowy pozbawiony zinterpretowanej czci.
   */
  private String FOR(String warunek, String kod){
    String[] w = warunek.toLowerCase().split("to");
    instrukcja(w[0].trim());
    String zmienna = ""+w[0].split(":=")[0];
    String war = zmienna+"<="+w[1];
    int loop = operacje.size();
    operacje.addElement("if");
    int koniec = operacje.size();
    operacje.addElement(" - tu bedzie adres konca ifa (ew. poczatek else) - ");
    wyrazenie(war);
    kod=kod.trim();
    if(kod.length()>5 && kod.substring(0,5).equalsIgnoreCase("begin")){ // Mamy blok operacji
      Blok b = dajBlokOperacji(kod);
//      if(wnetrze.charAt(wnetrze.length()-1)!=';') wnetrze+=";";
      wykonajBlok(b.blok);
      if(kod.length()>b.koniec+2) kod = kod.substring(b.koniec+2).trim();
      else  kod = kod.substring(b.koniec).trim();
    } else { // Mamy pojedycz operacj
      kod = wiersz(kod);
    }
// Zwiekszamy licznik o jeden
    operacje.addElement(":=");
    operacje.addElement("$"+indexZmiennej(zmienna));
    operacje.addElement("+");
    operacje.addElement("$"+indexZmiennej(zmienna));
    operacje.addElement("1");
    operacje.addElement("jmp");
    operacje.addElement(""+loop);
    operacje.setElementAt(String.valueOf(operacje.size()), koniec); // zaznaczenie konca if i poczatku ew. else
    return kod;
  }

  /**
   * Metoda interpretujca procedur <code>write(...)</code>
   *
   * @param co acuch znakw bdcy parametrem instrukcji <code>write</code>
   */
  private void write(String co){
    int lp=liczbaPlusow(co);
    if(lp>0){
      for(int i=0; i<lp; i++){
        int poz = co.indexOf("+");
        if(!czyWStringu(co, poz)){
          operacje.addElement("write");
          String lewa = co.substring(0, poz);
          String prawa = co.substring(poz+1);
          int zmienna = indexZmiennej(lewa);
          if(zmienna<0) operacje.addElement(dajString(lewa)); //Jaki text
          else operacje.addElement("$"+zmienna);   //Jaka zmienna
          write(prawa);
        } else {
          operacje.addElement("write");
          operacje.addElement(dajString(co));
        }
      }
    } else {
      operacje.addElement("write");
      int zmienna = indexZmiennej(co);
      if(zmienna<0) operacje.addElement(dajString(co)); //Jaki text
      else operacje.addElement("$"+zmienna);   //Jaka zmienna
    }
  }

  /**
   * Metoda interpretujca procedur <code>writeln(...)</code>
   *
   * @param co acuch znakw bdcy parametrem instrukcji <code>writeln</code>
   */
  private void writeln(String co){
    int lp=liczbaPlusow(co);
    if(lp>0){
      for(int i=0; i<lp; i++){
        int poz = co.indexOf("+");
        if(!czyWStringu(co, poz)){
          operacje.addElement("write");
          String lewa = co.substring(0, poz);
          String prawa = co.substring(poz+1);
          int zmienna = indexZmiennej(lewa);
          if(zmienna<0) operacje.addElement(dajString(lewa)); //Jaki text
          else operacje.addElement("$"+zmienna);   //Jaka zmienna
          writeln(prawa);
          break;
        } else {
          operacje.addElement("writeln");
          operacje.addElement(dajString(co));
        }
      }
    } else {
      operacje.addElement("writeln");
      int zmienna = indexZmiennej(co);
      if(zmienna<0) operacje.addElement(dajString(co)); //Jaki text
      else operacje.addElement("$"+zmienna);   //Jaka zmienna
    }
  }

  /**
   * Metoda interpretujca procedur <code>read(...)</code>
   *
   * @param co acuch znakw bdcy parametrem instrukcji <code>read</code>
   */
  private void read(String co){
    operacje.addElement("read");
    operacje.addElement("$"+indexZmiennej(co));
  }

  /**
   * Metoda interpretujca procedur <code>readln(...)</code>
   *
   * @param co acuch znakw bdcy parametrem instrukcji <code>readln</code>
   */
  private void readln(String co){
    operacje.addElement("readln");
    operacje.addElement("$"+indexZmiennej(co));
  }

  /**
   * Metoda sprawdza istnienie zmiennej o nazwie <code>nazwa</code> w tablicy
   * zadeklarowanych zmienney i w przypadku jej istnienia zwraca jej indeks w wektorze
   * lub -1 w przypadku jej braku
   *
   * @param nazwa nazwa zmiennej
   * @return indeks w wektorze zmiennych lub -1 w przypadku gdy zmienna nie znajduje si w wektorze zmiennych
   */
  private int indexZmiennej(String nazwa){
    int wy=-1;
    for(int i=0; i<zmienne.size(); i++){
      Zmienna z = (Zmienna)zmienne.elementAt(i);
      if (z.dajNazwe().equalsIgnoreCase(nazwa)) wy=i;
    };
    return wy;
  }

  /**
   * Metoda zwraca mniejsz z liczb A i B pod warunkiem e s wiksze od C
   *
   * @param A pierwsza liczba do porwnania
   * @param B druga liczba do porwnania
   * @param C minimalna liczba
   * @return mniejsza z liczb A i B pod warunkiem e s wiksze od C
   */
  private int min (int A, int B, int C){
    if(A<=C && B<=C) return C;
    if(A<=C) return B;
    if(B<=C) return A;
    if(A<B) return A;
    else return B;
  }

   /**
   * Metoda zwraca wiksz z dwch liczb bdcych parametrami
   *
   * @param A pierwsza liczba
   * @param B druga liczba
   * @return wiksza z dwch liczb bdcych parametrami
   */
  private int max (int A, int B){
    if(A>B) return A;
    else return B;
  }

  /**
   * Metoda wykonuje sekwencj krokw majcych na celu stworzenie pseudokodu dla caego kodu rdowego
   * znajdujcego si wewntrz klasy
   */
  public void run(){
    usunKomentarze();
    usunWskazniki();
    dajZmienne();
    dajStale();
    normalizujElse();
    String blokGlowny=dajBlokGlowny();
    wykonajBlok(blokGlowny);
  }

  /**
   * Metoda zwraca wektor zmiennych
   *
   * @return wektor zmiennych
   */
  public Vector Zmienne(){
    return zmienne;
  }

  /**
   * Metoda zwraca wektor operacji (pseudokod)
   *
   * @return wektor operacji
   */
  public Vector Operacje(){
    return operacje;
  }

}