package javapas;

/**
 * <p>Title: Javapas</p>
 * <p>Description: Pascal Interpreter</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: AP</p>
 * @author Artur Niewiadomski
 * @version 1.0
 */

import java.util.*;
import java.io.*;
/**
Klasa reprezentujca maszyn wirtualn systemu. Symuluje dziaanie prostego procesora. Dziaa na wektorze zawierajcym skompilowany pseudokod programu oraz wektorze zawierajcym zmienne.
*/
public class VM
{	/**
	Reprezentuje licznik instrucji w procesorze. Wskazuje na adres instrukcji w wektorze z pseudokodem programu, ktra ma by wykonana w nastpnej kolejnoci. UWAGA !!! Wywoanie konstruktora VM powoduje uruchomienie od razu kodu programu.
	*/
        int instr;
	/** Wektor zawierajcy pseudokod programu. Zawiera on zarwno instrukcje jak i ich argumenty oraz odwoania do wektora zmiennych.*/
        Vector v;
	/** Wektor zmiennych. Zawiera obiekty klasy Zmienna.*/
        Vector zm;
	/** Stos zawierajcy w trakcie wykonania programu czstkowe wyniki operacji.*/
        Stack stos=new Stack();
	/** Stos pomocniczy zawierajcy adresy skokw.*/
        Stack powrot=new Stack();
	/** Konstruktor tworzcy maszyn wirtualn z przykadowym kodem programu. */
        public VM(){
                v=new Vector();
                zm=new Vector();
                createVec();
                createZm();
                runCode();
        }
	/** Konstruktor tworzcy maszyn wirtualn z zadanymi parametrami wektorami.
         @param vect wektor zawierajcy kod programu,
         @param zmienne wektor zawierajacy zmienne wystpujce w kodzie.*/
        public VM(Vector vect, Vector zmienne){
                v=vect;
                zm=zmienne;
                runCode();
        }
	/** Uruchamia wykonywanie kodu przez maszyn wirtualn.
         Zawiera gwn ptl wykonujc instrukcje.
         Na pocztku wywoania licznik instrukcji jest ustawiany na zero.
         Metoda koczy dziaanie gdy licznik instrukcji dojdzie do koca wektora z instrukcjami.*/
        public void runCode(){
                instr=0;
                while(instr<v.size()){	//petla glowna
                try{
                        instr=wykonajInstrukcje(instr);// instr wskazuje na instrukcje
                }catch (Throwable e) {e.printStackTrace(); System.exit(1);}

                }//koniec petli glownej
        }
        /**Wykonuje pojedyncz instrukcj pseudokodu.
         Rozpoznawany jest rodzaj instrukcji, a nastpnie wywoywana jest odpowiednia metoda obsugujca
         dany rodzaj instrukcji ze wskazaniem indeksu pierwszego z parametrw instrukcji.
         Wyjtkiem s instrukcje write i writeln, ktre s obsugiwane w ciele metody.
         Wywoywane metody obsugujce instrukcje mog wyrzuca wyjtki klasy Throwable.
         @param NR numer instrukcji do wykonania, jest to indeks instrukcji w wektorze instrukcji (v).
         @return Funkcja zwraca indeks nastpnej instrukcji, ktra ma by wykonana.
         @throws Throwable ewentualne wyjtki s wyrzucane wyej.
         */
        int wykonajInstrukcje(int NR) throws Throwable{
                int i=NR+1;
                String s;
                String line=(String)v.get(NR);

                if(line.equals("write")){
                        s=(String)v.get(i);
                        if(czyZmienna(s)) {
                          if(getVar(s).dajTyp().equalsIgnoreCase("string"))
                            s=getVar(s).dajWartosc().substring(1, getVar(s).dajWartosc().length()-1);
                          else
                            s=getVar(s).dajWartosc();
                        }
			if(s.endsWith(".0")) s=s.substring(0,s.length()-2);
                        System.out.print(s);
                        i++;
                        return i;
                }

                if(line.equals("writeln")){
                        s=(String)v.get(i);
                        if(czyZmienna(s)) {
                          if(getVar(s).dajTyp().equalsIgnoreCase("string"))
                            s=getVar(s).dajWartosc().substring(1, getVar(s).dajWartosc().length()-1);
                          else
                            s=getVar(s).dajWartosc();
                        }
			if(s.endsWith(".0")) s=s.substring(0,s.length()-2);
                        System.out.println(s);
                        i++;
                        return i;
                }

				if(line.equals("read")){
					return READ(i);
				}

				if(line.equals("readln")){
					return READLN(i);
				}


                if(line.equals(":=")){// podstawienie
                        return podstawienie(i);// i wskazuje na argument
                }

                if(line.equals("+")){
                        return dodaj(i);// i wskazuje na argument
                }
                if(line.equals("*")){
                        return mnozenie(i);// i wskazuje na argument
                }
                if(line.equals("/")){
                        return podziel(i);// i wskazuje na argument
                }
                if(line.equals("-")){
                        return minus(i);// i wskazuje na argument
                }
                if(line.equals("and")){
                        return AND(i);// i wskazuje na argument
                }
                if(line.equals("or")){
                        return OR(i);// i wskazuje na argument
                }
                if(line.equals(">")){
                        return wieksze(i);// i wskazuje na argument
                }
                if(line.equals("<")){
                        return mniejsze(i);// i wskazuje na argument
                }
                if(line.equals("=")){
                        return rowne(i);// i wskazuje na argument
                }
                if(line.equals(">=")){
                        return wiekszeRowne(i);// i wskazuje na argument
                }
                if(line.equals("<=")){
                        return mniejszeRowne(i);// i wskazuje na argument
                }
                if(line.equals("<>")){
                        return rozne(i);// i wskazuje na argument
                }
                if(line.equals("jmp")){
                        return (new Integer((String)v.get(i))).intValue();
                }
                if(line.equals("if")){
                        return IF(i);// i wskazuje na argument
                }

                return i;
        }
//-------------------------------------------------------------------
        /** Funkcja obsugujca instrukcj READ pseudokodu odpowiedzialn za wczytywanie
         * danych z klawiatury.
         * @param i indeks parametru w wektorze instrukcji. Wektor v powinien w tym miejscu zawiera
         * zmienn, na ktr ma zosta wczytana warto z klawiatury.
         * @return Funkcja zwraca adres (indeks) nastpnej instrukcji w wektorze instrukcji.
         * */
int READ(int i){// i wskazuje na argument - zmienna na ktora wczytac
	try{
	BufferedReader in=new BufferedReader(new InputStreamReader(System.in));

	Zmienna zm=getVar((String)v.get(i));
	zm.ustawWartosc(in.readLine());
	}catch(IOException e){e.printStackTrace();}
	i++;
	return i;
}
/** Funkcja obsugujca instrukcj READLN pseudokodu odpowiedzialn za wczytywanie
         * danych z klawiatury.
         * @param i indeks parametru w wektorze instrukcji. Wektor v powinien w tym miejscu zawiera
         * zmienn, na ktr ma zosta wczytana warto z klawiatury.
         * @return Funkcja zwraca adres (indeks) nastpnej instrukcji w wektorze instrukcji.
         * */
int READLN(int i){
	BufferedReader in=new BufferedReader(new InputStreamReader(System.in));
	try{
	Zmienna zm=getVar((String)v.get(i));
	zm.ustawWartosc(in.readLine());
	}catch(IOException e){e.printStackTrace();}
	i++;
	return i;
}

//---------------------------------------------------------------------
/** Metoda obsugujca instrukcj if pseudokodu. Instrukcja if ma dwa agumenty.
 * Najpierw pobierany jest argument tu za instrukcj. Jest to adres pierwszej instrukcji
 * za blokiem instrukcji wentrz if. Adres ten zostaje umieszczony na stosie powrot.
 * Nastpnie pobierany jest drugi argument, ktry w najprostszym przypadku jest wartoci
 * lub zmienn logiczn. Jeli warto ta jest prawd, nastpuje zdjcie ze stosu uprzednio
 * umieszczonego tam adresu, gdy nie bdzie on wykorzystany w tym kroku. Nastpnie zwracany jest
 * adres (indeks) nastpnej instrukcji (o jeden wikszy ni drugiego argumentu).
 * Drugim argumentem moe rwnie by operator logiczny. Wtedy wywoywana jest metoda
 * wykonajInstrukcje z indeksem tego operatora. Nastpnie w wyniku cigu rekurencyjnych wywoa
 * funkcji (po powrocie do aktualnego wywoania funkcji IF) na stosie stos
 * jest obliczona warto wyraenia logicznego.
 * Dalej wszystko dzieje si analogicznie do przypadku pierwszego.
 * @param i indeks pierwszego parametru dla instrukcji if.
 * @return Zwraca indeks nastpnej instrukcji do wykonania.
 * @throws Throwable ewentualne wyjtki wyrzyca wyej.
 */

        int IF(int i) throws Throwable{
                powrot.push(new Integer((String)v.get(i)));
                i++;
                String l1;
                Boolean b1;
                l1=(String)v.get(i);
                if(czyOperator(l1)){
                        i=wykonajInstrukcje(i);
                        b1=new Boolean((String)stos.pop());
                }
                else if(czyZmienna(l1)) b1=new Boolean(getVar(l1).dajWartosc());
                          else b1=new Boolean(l1);
                i++;
                if(! b1.booleanValue())
                        return ((Integer)powrot.pop()).intValue();

                powrot.pop();
                return i;

        }

//-------------------------------------------------------------------------
/** Metoda obsugujca operator dodawania ( + ).
 * Jest to operator dwuargumentowy. Najpierw pobierany jest pierwszy argument.
 * Jeli pierwszy argument jest innym operatorem (np. mnoenia), oznacza to e mamy
 * do czynienia z bardziej skomplikowanym wyraeniem arytmetycznym. Wtedy wywoywana jest
 * metoda wykonajInstrukcje w celu ewaluacji tego podwyraenia.
 * Po powrocie z cigu wywoa funkcji obliczona warto podwyraenia znajduje si na stosie stos.
 * Podobnie dzieje si z drugim argumentem. Kiedy mamy ju obydwa argumenty
 * s one do siebie dodawane, a wynik zostaje pooony na stosie stos.
 * @param i indeks pierwszego argumentu w wektorze v.
 * @return Zwraca indeks nastpnej instrukcji w wektorze v.
 * @throws Throwable ewentualne wyjtki wyrzuca wyej.
 */

        int dodaj(int i) throws Throwable{// i wskazuje na argument
                String l1,l2;
                Zmienna z1,z2;
                int pom1,pom2;
                Double d1,d2;
                l1=(String)v.get(i);
                if(czyOperator(l1)){
                        i=wykonajInstrukcje(i);
                        d1=new Double((String)stos.pop());
                }
                else if(czyZmienna(l1)) d1=new Double(getVar(l1).dajWartosc());
                          else d1=new Double(l1);
                i++;
                l2=(String)v.get(i);
                if(czyOperator(l2)){
                        i=wykonajInstrukcje(i);
                        //System.out.println(stos.peek());
                        d2=new Double((String)stos.pop());
                }
                else if(czyZmienna(l2)) d2=new Double(getVar(l2).dajWartosc());
                          else d2=new Double(l2);

                stos.push((new Double(d1.doubleValue()+d2.doubleValue())).toString());
                return i++;
        }

    /** Metoda obsugujca operator dzielenia ( / ).
     * Jest to operator dwuargumentowy. Najpierw pobierany jest pierwszy argument.
     * Jeli pierwszy argument jest innym operatorem (np. dodawania), oznacza to e mamy
     * do czynienia z bardziej skomplikowanym wyraeniem arytmetycznym. Wtedy wywoywana jest
     * metoda wykonajInstrukcje w celu ewaluacji tego podwyraenia.
     * Po powrocie z cigu wywoa funkcji obliczona warto podwyraenia znajduje si na stosie stos.
     * Podobnie dzieje si z drugim argumentem. Kiedy mamy ju obydwa argumenty
     * pierszy jest dzielony przez drugi, a wynik zostaje pooony na stosie stos.
     * @param i indeks pierwszego argumentu w wektorze v.
     * @return Zwraca indeks nastpnej instrukcji w wektorze v.
     * @throws Throwable jeli dzielnik jest zerem.
 */
        int podziel(int i) throws Throwable{// i wskazuje na argument
                String l1,l2;
                Zmienna z1,z2;
                int pom1,pom2;
                Double d1,d2;
                l1=(String)v.get(i);
                if(czyOperator(l1)){
                        i=wykonajInstrukcje(i);
                        d1=new Double((String)stos.pop());
                }
                else if(czyZmienna(l1)) d1=new Double(getVar(l1).dajWartosc());
                          else d1=new Double(l1);
                i++;
                l2=(String)v.get(i);
                if(czyOperator(l2)){
                        i=wykonajInstrukcje(i);
                        //System.out.println(stos.peek());
                        d2=new Double((String)stos.pop());
                }
                else if(czyZmienna(l2)) d2=new Double(getVar(l2).dajWartosc());
                          else d2=new Double(l2);

                if(d2.doubleValue()==0) throw (Throwable)
                  new Throwable("Prba dzielenia przez zero.");

                stos.push((new Double(d1.doubleValue()/d2.doubleValue())).toString());
                return i++;
        }

    /** Metoda obsugujca operator odejmowania ( - ).
     * Jest to operator dwuargumentowy. Najpierw pobierany jest pierwszy argument.
     * Jeli pierwszy argument jest innym operatorem (np. mnoenia), oznacza to e mamy
     * do czynienia z bardziej skomplikowanym wyraeniem arytmetycznym. Wtedy wywoywana jest
     * metoda wykonajInstrukcje w celu ewaluacji tego podwyraenia.
     * Po powrocie z cigu wywoa funkcji obliczona warto podwyraenia znajduje si na stosie stos.
     * Podobnie dzieje si z drugim argumentem. Kiedy mamy ju obydwa argumenty
     * od pierwszego zostaje odjty drugi, a wynik zostaje pooony na stosie stos.
     * @param i indeks pierwszego argumentu w wektorze v.
     * @return Zwraca indeks nastpnej instrukcji w wektorze v.
     * @throws Throwable ewentualne wyjtki wyrzuca wyej.
 */
        int minus(int i) throws Throwable{// i wskazuje na argument
                String l1,l2;
                Zmienna z1,z2;
                int pom1,pom2;
                Double d1,d2;
                l1=(String)v.get(i);
                if(czyOperator(l1)){
                        i=wykonajInstrukcje(i);
                        d1=new Double((String)stos.pop());
                }
                else if(czyZmienna(l1)) d1=new Double(getVar(l1).dajWartosc());
                          else d1=new Double(l1);
                i++;
                l2=(String)v.get(i);
                if(czyOperator(l2)){
                        i=wykonajInstrukcje(i);
                        //System.out.println(stos.peek());
                        d2=new Double((String)stos.pop());
                }
                else if(czyZmienna(l2)) d2=new Double(getVar(l2).dajWartosc());
                          else d2=new Double(l2);

                stos.push((new Double(d1.doubleValue()-d2.doubleValue())).toString());
                return i++;
        }
            /** Metoda obsugujca operator mnozenia ( * ).
             * Jest to operator dwuargumentowy. Najpierw pobierany jest pierwszy argument.
             * Jeli pierwszy argument jest innym operatorem (np. dodawania), oznacza to e mamy
             * do czynienia z bardziej skomplikowanym wyraeniem arytmetycznym. Wtedy wywoywana jest
             * metoda wykonajInstrukcje w celu ewaluacji tego podwyraenia.
             * Po powrocie z cigu wywoa funkcji obliczona warto podwyraenia znajduje si na stosie stos.
             * Podobnie dzieje si z drugim argumentem. Kiedy mamy ju obydwa argumenty
             * s one przez siebie mnoone, a wynik zostaje pooony na stosie stos.
             * @param i indeks pierwszego argumentu w wektorze v.
             * @return Zwraca indeks nastpnej instrukcji w wektorze v.
             * @throws Throwable ewentualne wyjtki wyrzuca wyej.
 */
        int mnozenie(int i) throws Throwable{// i wskazuje na argument
                String l1,l2;
                Zmienna z1,z2;
                int pom1,pom2;
                Double d1,d2;
                l1=(String)v.get(i);
                if(czyOperator(l1)){
                        i=wykonajInstrukcje(i);
                        d1=new Double((String)stos.pop());
                }
                else if(czyZmienna(l1)) d1=new Double(getVar(l1).dajWartosc());
                          else d1=new Double(l1);
                i++;
                l2=(String)v.get(i);
                if(czyOperator(l2)){
                        i=wykonajInstrukcje(i);
                        //System.out.println(stos.peek());
                        d2=new Double((String)stos.pop());
                }
                else if(czyZmienna(l2)) d2=new Double(getVar(l2).dajWartosc());
                          else d2=new Double(l2);

                stos.push((new Double(d1.doubleValue()*d2.doubleValue())).toString());
                return i++;
        }
//----------------------------------------------------------------------------
    /** Metoda obsugujca operator logiczny and.
     * Jest to operator dwuargumentowy. Najpierw pobierany jest pierwszy argument.
     * Jeli pierwszy argument jest innym operatorem (np. wikszoci), oznacza to e mamy
     * do czynienia z bardziej skomplikowanym wyraeniem logicznym. Wtedy wywoywana jest
     * metoda wykonajInstrukcje w celu ewaluacji tego podwyraenia.
     * Po powrocie z cigu wywoa funkcji obliczona warto podwyraenia znajduje si na stosie stos.
     * Podobnie dzieje si z drugim argumentem. Kiedy mamy ju obydwa argumenty
     * s one poddane operacji logicznej and ( && ), a wynik zostaje pooony na stosie stos.
     * @param i indeks pierwszego argumentu w wektorze v.
     * @return Zwraca indeks nastpnej instrukcji w wektorze v.
     * @throws Throwable ewentualne wyjtki wyrzuca wyej.
 */
int AND(int i) throws Throwable{// i wskazuje na argument
                String l1,l2;
                Zmienna z1,z2;
                int pom1,pom2;
                Boolean b1,b2;
                l1=(String)v.get(i);
                if(czyOperator(l1)){
                        i=wykonajInstrukcje(i);
                        b1=new Boolean((String)stos.pop());
                }
                else if(czyZmienna(l1)) b1=new Boolean(getVar(l1).dajWartosc());
                          else b1=new Boolean(l1);
                i++;
                l2=(String)v.get(i);
                if(czyOperator(l2)){
                        i=wykonajInstrukcje(i);
                        //System.out.println(stos.peek());
                        b2=new Boolean((String)stos.pop());
                }
                else if(czyZmienna(l2)) b2=new Boolean(getVar(l2).dajWartosc());
                          else b2=new Boolean(l2);
                stos.push(
                        (new Boolean(b1.booleanValue() && b2.booleanValue())).toString());
                return i++;
        }
        /** Metoda obsugujca operator logiczny OR.
             * Jest to operator dwuargumentowy. Najpierw pobierany jest pierwszy argument.
             * Jeli pierwszy argument jest innym operatorem (np. wikszoci), oznacza to e mamy
             * do czynienia z bardziej skomplikowanym wyraeniem logicznym. Wtedy wywoywana jest
             * metoda wykonajInstrukcje w celu ewaluacji tego podwyraenia.
             * Po powrocie z cigu wywoa funkcji obliczona warto podwyraenia znajduje si na stosie stos.
             * Podobnie dzieje si z drugim argumentem. Kiedy mamy ju obydwa argumenty
             * s one poddane operacji logicznej OR ( || ), a wynik zostaje pooony na stosie stos.
             * @param i indeks pierwszego argumentu w wektorze v.
             * @return Zwraca indeks nastpnej instrukcji w wektorze v.
             * @throws Throwable ewentualne wyjtki wyrzuca wyej.
 */
        int OR(int i) throws Throwable{// i wskazuje na argument
                String l1,l2;
                Zmienna z1,z2;
                int pom1,pom2;
                Boolean b1,b2;
                l1=(String)v.get(i);
                if(czyOperator(l1)){
                        i=wykonajInstrukcje(i);
                        b1=new Boolean((String)stos.pop());
                }
                else if(czyZmienna(l1)) b1=new Boolean(getVar(l1).dajWartosc());
                          else b1=new Boolean(l1);
                i++;
                l2=(String)v.get(i);
                if(czyOperator(l2)){
                        i=wykonajInstrukcje(i);
                        //System.out.println(stos.peek());
                        b2=new Boolean((String)stos.pop());
                }
                else if(czyZmienna(l2)) b2=new Boolean(getVar(l2).dajWartosc());
                          else b2=new Boolean(l2);
                stos.push(
                        (new Boolean(b1.booleanValue() || b2.booleanValue())).toString());
                return i++;
        }
//-----------------------------------------------------------------------------
/** Metoda obsugujca operator wikszoci ( > ).
     * Jest to operator dwuargumentowy. Najpierw pobierany jest pierwszy argument.
     * Jeli pierwszy argument jest innym operatorem (np. dodawania), oznacza to e mamy
     * do czynienia z bardziej skomplikowanym wyraeniem. Wtedy wywoywana jest
     * metoda wykonajInstrukcje w celu ewaluacji tego podwyraenia.
     * Po powrocie z cigu wywoa funkcji obliczona warto podwyraenia znajduje si na stosie stos.
     * Podobnie dzieje si z drugim argumentem. Kiedy mamy ju obydwa argumenty
     * s one ze sob porwnywane, a wynik zostaje pooony na stosie stos.
     * @param i indeks pierwszego argumentu w wektorze v.
     * @return Zwraca indeks nastpnej instrukcji w wektorze v.
     * @throws Throwable ewentualne wyjtki wyrzuca wyej.
 */
int wieksze(int i) throws Throwable{// i wskazuje na argument
                String l1,l2;
                Zmienna z1,z2;
                int pom1,pom2;
                Double d1,d2;
                l1=(String)v.get(i);
                if(czyOperator(l1)){
                        i=wykonajInstrukcje(i);
                        d1=new Double((String)stos.pop());
                }
                else if(czyZmienna(l1)) d1=new Double(getVar(l1).dajWartosc());
                          else d1=new Double(l1);
                i++;
                l2=(String)v.get(i);
                if(czyOperator(l2)){
                        i=wykonajInstrukcje(i);
                        //System.out.println(stos.peek());
                        d2=new Double((String)stos.pop());
                }
                else if(czyZmienna(l2)) d2=new Double(getVar(l2).dajWartosc());
                          else d2=new Double(l2);

                stos.push((new Boolean(d1.doubleValue()>d2.doubleValue())).toString());
                return i++;
        }
        /** Metoda obsugujca operator mniejszoci ( &lt; ).
         * Jest to operator dwuargumentowy. Najpierw pobierany jest pierwszy argument.
         * Jeli pierwszy argument jest innym operatorem (np. dodawania), oznacza to e mamy
         * do czynienia z bardziej skomplikowanym wyraeniem. Wtedy wywoywana jest
         * metoda wykonajInstrukcje w celu ewaluacji tego podwyraenia.
         * Po powrocie z cigu wywoa funkcji obliczona warto podwyraenia znajduje si na stosie stos.
         * Podobnie dzieje si z drugim argumentem. Kiedy mamy ju obydwa argumenty
         * s one ze sob porwnywane, a wynik zostaje pooony na stosie stos.
         * @param i indeks pierwszego argumentu w wektorze v.
         * @return Zwraca indeks nastpnej instrukcji w wektorze v.
         * @throws Throwable ewentualne wyjtki wyrzuca wyej.
         * */
        int mniejsze(int i) throws Throwable{// i wskazuje na argument
                String l1,l2;
                Zmienna z1,z2;
                int pom1,pom2;
                Double d1,d2;
                l1=(String)v.get(i);
                if(czyOperator(l1)){
                        i=wykonajInstrukcje(i);
                        d1=new Double((String)stos.pop());
                }
                else if(czyZmienna(l1)) d1=new Double(getVar(l1).dajWartosc());
                          else d1=new Double(l1);
                i++;
                l2=(String)v.get(i);
                if(czyOperator(l2)){
                        i=wykonajInstrukcje(i);
                        //System.out.println(stos.peek());
                        d2=new Double((String)stos.pop());
                }
                else if(czyZmienna(l2)) d2=new Double(getVar(l2).dajWartosc());
                          else d2=new Double(l2);

                stos.push((new Boolean(d1.doubleValue()<d2.doubleValue())).toString());
                return i++;
        }
        /** Metoda obsugujca operator rwnoci ( = ).
             * Jest to operator dwuargumentowy. Najpierw pobierany jest pierwszy argument.
             * Jeli pierwszy argument jest innym operatorem (np. dodawania), oznacza to e mamy
             * do czynienia z bardziej skomplikowanym wyraeniem. Wtedy wywoywana jest
             * metoda wykonajInstrukcje w celu ewaluacji tego podwyraenia.
             * Po powrocie z cigu wywoa funkcji obliczona warto podwyraenia znajduje si na stosie stos.
             * Podobnie dzieje si z drugim argumentem. Kiedy mamy ju obydwa argumenty
             * s one ze sob porwnywane, a wynik zostaje pooony na stosie stos.
             * @param i indeks pierwszego argumentu w wektorze v.
             * @return Zwraca indeks nastpnej instrukcji w wektorze v.
             * @throws Throwable ewentualne wyjtki wyrzuca wyej.
 */
        int rowne(int i) throws Throwable{// i wskazuje na argument
                String l1,l2;
                Zmienna z1,z2;
                int pom1,pom2;
                Double d1,d2;
                l1=(String)v.get(i);
                if(czyOperator(l1)){
                        i=wykonajInstrukcje(i);
                        d1=new Double((String)stos.pop());
                }
                else if(czyZmienna(l1)) d1=new Double(getVar(l1).dajWartosc());
                          else d1=new Double(l1);
                i++;
                l2=(String)v.get(i);
                if(czyOperator(l2)){
                        i=wykonajInstrukcje(i);
                        //System.out.println(stos.peek());
                        d2=new Double((String)stos.pop());
                }
                else if(czyZmienna(l2)) d2=new Double(getVar(l2).dajWartosc());
                          else d2=new Double(l2);

                stos.push((new Boolean(d1.doubleValue() == d2.doubleValue())).toString());
                return i++;
        }
        /** Metoda obsugujca operator "wiksze bd rwne" ( &gt;= ).
             * Jest to operator dwuargumentowy. Najpierw pobierany jest pierwszy argument.
             * Jeli pierwszy argument jest innym operatorem (np. dodawania), oznacza to e mamy
             * do czynienia z bardziej skomplikowanym wyraeniem. Wtedy wywoywana jest
             * metoda wykonajInstrukcje w celu ewaluacji tego podwyraenia.
             * Po powrocie z cigu wywoa funkcji obliczona warto podwyraenia znajduje si na stosie stos.
             * Podobnie dzieje si z drugim argumentem. Kiedy mamy ju obydwa argumenty
             * s one ze sob porwnywane, a wynik zostaje pooony na stosie stos.
             * @param i indeks pierwszego argumentu w wektorze v.
             * @return Zwraca indeks nastpnej instrukcji w wektorze v.
             * @throws Throwable ewentualne wyjtki wyrzuca wyej.
 */
        int wiekszeRowne(int i) throws Throwable{// i wskazuje na argument
                String l1,l2;
                Zmienna z1,z2;
                int pom1,pom2;
                Double d1,d2;
                l1=(String)v.get(i);
                if(czyOperator(l1)){
                        i=wykonajInstrukcje(i);
                        d1=new Double((String)stos.pop());
                }
                else if(czyZmienna(l1)) d1=new Double(getVar(l1).dajWartosc());
                          else d1=new Double(l1);
                i++;
                l2=(String)v.get(i);
                if(czyOperator(l2)){
                        i=wykonajInstrukcje(i);
                        //System.out.println(stos.peek());
                        d2=new Double((String)stos.pop());
                }
                else if(czyZmienna(l2)) d2=new Double(getVar(l2).dajWartosc());
                          else d2=new Double(l2);

                stos.push((new Boolean(d1.doubleValue()>=d2.doubleValue())).toString());
                return i++;
        }
      /** Metoda obsugujca operator "mniejsze bd rwne" ( &lt;= ).
        * Jest to operator dwuargumentowy. Najpierw pobierany jest pierwszy argument.
        * Jeli pierwszy argument jest innym operatorem (np. dodawania), oznacza to e mamy
        * do czynienia z bardziej skomplikowanym wyraeniem. Wtedy wywoywana jest
        * metoda wykonajInstrukcje w celu ewaluacji tego podwyraenia.
        * Po powrocie z cigu wywoa funkcji obliczona warto podwyraenia znajduje si na stosie stos.
        * Podobnie dzieje si z drugim argumentem. Kiedy mamy ju obydwa argumenty
        * s one ze sob porwnywane, a wynik zostaje pooony na stosie stos.
        * @param i indeks pierwszego argumentu w wektorze v.
        * @return Zwraca indeks nastpnej instrukcji w wektorze v.
        * @throws Throwable ewentualne wyjtki wyrzuca wyej.
 */
        int mniejszeRowne(int i) throws Throwable{// i wskazuje na argument
                String l1,l2;
                Zmienna z1,z2;
                int pom1,pom2;
                Double d1,d2;
                l1=(String)v.get(i);
                if(czyOperator(l1)){
                        i=wykonajInstrukcje(i);
                        d1=new Double((String)stos.pop());
                }
                else if(czyZmienna(l1)) d1=new Double(getVar(l1).dajWartosc());
                          else d1=new Double(l1);
                i++;
                l2=(String)v.get(i);
                if(czyOperator(l2)){
                        i=wykonajInstrukcje(i);
                        //System.out.println(stos.peek());
                        d2=new Double((String)stos.pop());
                }
                else if(czyZmienna(l2)) d2=new Double(getVar(l2).dajWartosc());
                          else d2=new Double(l2);

                stos.push((new Boolean(d1.doubleValue()<=d2.doubleValue())).toString());
                return i++;
        }
        /** Metoda obsugujca operator "rne od" ( &lt;&gt; ).
                * Jest to operator dwuargumentowy. Najpierw pobierany jest pierwszy argument.
                * Jeli pierwszy argument jest innym operatorem (np. dodawania), oznacza to e mamy
                * do czynienia z bardziej skomplikowanym wyraeniem. Wtedy wywoywana jest
                * metoda wykonajInstrukcje w celu ewaluacji tego podwyraenia.
                * Po powrocie z cigu wywoa funkcji obliczona warto podwyraenia znajduje si na stosie stos.
                * Podobnie dzieje si z drugim argumentem. Kiedy mamy ju obydwa argumenty
                * s one ze sob porwnywane, a wynik zostaje pooony na stosie stos.
                * @param i indeks pierwszego argumentu w wektorze v.
                * @return Zwraca indeks nastpnej instrukcji w wektorze v.
                * @throws Throwable ewentualne wyjtki wyrzuca wyej.
 */
        int rozne(int i) throws Throwable{// i wskazuje na argument
                String l1,l2;
                Zmienna z1,z2;
                int pom1,pom2;
                Double d1,d2;
                l1=(String)v.get(i);
                if(czyOperator(l1)){
                        i=wykonajInstrukcje(i);
                        d1=new Double((String)stos.pop());
                }
                else if(czyZmienna(l1)) d1=new Double(getVar(l1).dajWartosc());
                          else d1=new Double(l1);
                i++;
                l2=(String)v.get(i);
                if(czyOperator(l2)){
                        i=wykonajInstrukcje(i);
                        //System.out.println(stos.peek());
                        d2=new Double((String)stos.pop());
                }
                else if(czyZmienna(l2)) d2=new Double(getVar(l2).dajWartosc());
                          else d2=new Double(l2);

                stos.push((new Boolean(d1.doubleValue() != d2.doubleValue())).toString());
                return i++;
        }
//------------------------------------------------------------------------
/** Metoda obsugujca operator postawienia ( := ).
 *  Jest to operator dwuargumentowy. Pierwszym argumentem jest zmienna, na ktr
 *  ma by podstawiona warto, a drugim wyraenie, zmienna lub warto.
 *  Jeli jest to warto lub zmienna, to zostaje ustawiona odpowiednia warto w wektorze zmiennych.
 *  Jeli jest to wyraenie, to najpierw zostaje wywoana funkcja wykonajInstrukcje w celu
 *  obliczenia wartosci wyraenia. Nastpnie obliczona warto jest cigana ze stosu i podstawiana
 *  na odpowiedni zmienn w wektorze zmiennych.
 * @param i indeks pierwszego argumentu w wektorze v.
 * @return indeks nastpnej instrukcji w wektorze instrukcji (v).
 * @throws Throwable jeli pierszym argumentem nie jest zmienna.
 *
 */
        int podstawienie(int i) throws Throwable{// i wskazuje na argument
                String l1,l2;
                Zmienna z1,z2;
                int pom1,pom2;

                l1=(String)v.get(i);
                        //na wszelki wypadek
                        if(! czyZmienna(l1))
                                throw (Throwable)
                                new Throwable("Powinna byc zmienna, a nie: "+l1+" na pozycji "+i);

                z1=getVar(l1);
                //System.out.println("Zmienna: "+z1);
                i++;
                l2=(String)v.get(i);

                if(czyZmienna(l2)){
                        z2=getVar(l2);
                        z1.ustawWartosc(new String(z2.dajWartosc()));
                        i++;
                        return i;
                }
                if(czyOperator(l2)){
                        i=wykonajInstrukcje(i);
                        z1.ustawWartosc((String)stos.pop());
                        return i;
                }
                z1.ustawWartosc(l2);
                i++;
                return i;
        }

/** Sprawdza, czy dany cig znakw identyfikuje zmienn.
 *  Wszystkie cigi znakw reprezentujce zmienne zaczynaj si od znaku dolara ( $ ).
 *  @param str cig znakw poddawany testowi.
 *  @return true, jeli str reprezentuje zmienn lub false w przeciwnym przypadku.
 */
        private boolean czyZmienna(String str){
                return str.startsWith("$");
        }
/** @param str cig znakw identyfikujcy zmienn.
    @return Zwraca zmienn identyfikowan przez dany cig znakw.
  */
        private Zmienna getVar(String str){
                return (Zmienna)zm.get((new Integer(str.substring(1))).intValue());
        }
/** @param str cig znakw poddawanych testowi.
    @return true, jeli cig znakw jest operatorem, false jeli nim nie jest.
    */
        private boolean czyOperator(String str){
                boolean wynik=false;
                wynik=wynik || str.equals("+");
                wynik=wynik || str.equals("-");
                wynik=wynik || str.equals("*");
                wynik=wynik || str.equals("/");
                wynik=wynik || str.equals(">");
                wynik=wynik || str.equals("<");
                wynik=wynik || str.equals(">=");
                wynik=wynik || str.equals("<=");
                wynik=wynik || str.equals("=");
                wynik=wynik || str.equals("<>");
                wynik=wynik || str.equals("and");
                wynik=wynik || str.equals("or");
                return wynik;
        }
/** Tworzy wektor zawierajcy przykadowy kod programu.*/
        private void createVec(){
        //	v.add("writeln");
        //	v.add("dzialanie programu");
                v.add(":=");
                v.add("$0");
                v.add("0");
                v.add(":=");
                v.add("$1");
                v.add(">");
                v.add("+");
                v.add("3");
                v.add("-");
                v.add("*");
                v.add("5");
                v.add("2");
                v.add("1");
                v.add("10");
                v.add("if");
                v.add("29");
                v.add("and");
                v.add("$1");
                v.add(">");
                v.add("$0");
                v.add("2");
                v.add("write");
                v.add("jestem");
                v.add("writeln");
                v.add(" w ifie");
                v.add("writeln");
                v.add("$0");
                v.add("writeln");
                v.add("$1");
                v.add("writeln");
                v.add("jestem za ifem");
        }
/** Tworzy wektor zmiennych uywanych w przykadowym kodzie programu utworzonym
 za pomoc createVec.*/
        private void createZm(){
                zm.add(new Zmienna("abc","0","integer"));
                zm.add(new Zmienna("bool","false", "boolean"));
        }



}
